/*
 * SPDX-FileCopyrightText: 2011 Erick Pérez Castellanos <erick.red@gmail.com>
 * SPDX-FileCopyrightText: 2019 Florian Müllner <fmuellner@gnome.org>
 *
 * SPDX-License-Identifier: GPL-2.0-or-later
 */

.panel-workspace-indicator {
  padding: 0 8px;
}

.panel-workspace-indicator-box {
  padding: 4px 0;
  spacing: 4px;
}

.panel-workspace-indicator-box .workspace {
  width: 40px;
  border: 2px solid #000;
  border-radius: 2px;
  background-color: #595959;
}

.panel-workspace-indicator-box .workspace.active {
  border-color: #fff;
}

.workspace-indicator-window-preview {
  background-color: #bebebe;
  border: 1px solid #828282;
}

.workspace-indicator-window-preview.active {
  background-color: #d4d4d4;
}
