/*
 * SPDX-FileCopyrightText: 2013 Florian Müllner <fmuellner@gnome.org>
 *
 * SPDX-License-Identifier: GPL-2.0-or-later
 */

.apps-menu:ltr {
    padding-right: 3px;
}

.apps-menu:rtl {
    padding-left: 3px;
}
